import { FrameButton, Frame, TransactionTargetResponse } from 'frames.js';
import { CastId, FrameActionMessage } from '@farcaster/core';

type FarcasterSignerState = SignerStateInstance<FarcasterSigner | null>;
interface FarcasterSigner {
    privateKey: string;
    publicKey: string;
    fid?: number;
    /** The status of the signer */
    status: "approved" | "pending_approval" | "impersonating";
    signature?: string;
    deadline?: number;
    signerApprovalUrl?: string;
    token?: any;
}
declare const mockFarcasterSigner: FarcasterSigner;

type FarcasterFrameContext = {
    /** Connected address of user, only sent with transaction data request */
    connectedAddress?: `0x${string}`;
    castId: {
        hash: `0x${string}`;
        fid: number;
    };
};
/** Creates a frame action for use with `useFrame` and a proxy */
declare const signFrameAction: ({ buttonIndex, frameContext, frameButton, signer, target, inputText, state, transactionId, url, }: {
    target?: string | undefined;
    frameButton: FrameButton;
    buttonIndex: number;
    url: string;
    signer: FarcasterSignerState["signer"];
    inputText?: string | undefined;
    state?: string | undefined;
    transactionId?: `0x${string}` | undefined;
    frameContext: FrameContext;
}) => Promise<{
    body: object;
    searchParams: URLSearchParams;
}>;
declare function createFrameActionMessageWithSignerKey(signerKey: string, { fid, url, buttonIndex, castId, inputText, state, address, transactionId, }: {
    fid: number;
    url: Uint8Array;
    buttonIndex: number;
    inputText: Uint8Array | undefined;
    castId: CastId;
    state: Uint8Array | undefined;
    address: Uint8Array | undefined;
    transactionId: Uint8Array | undefined;
}): Promise<{
    message: null;
    trustedBytes: null;
} | {
    message: FrameActionMessage | null;
    trustedBytes: string;
}>;

type OnTransactionFunc = (t: OnTransactionArgs) => Promise<`0x${string}` | null>;
type UseFrameReturn<T = object, B extends FrameActionBodyPayload = FrameActionBodyPayload> = {
    /** skip frame signing, for frames that don't verify signatures */
    dangerousSkipSigning?: boolean;
    /** the route used to POST frame actions. The post_url will be added as a the `url` query parameter */
    frameActionProxy: string;
    /** the route used to GET the initial frame via proxy */
    frameGetProxy: string;
    /** an signer state object used to determine what actions are possible */
    signerState: SignerStateInstance<T, B>;
    /** the url of the homeframe, if null won't load a frame */
    homeframeUrl: string | null;
    /** the initial frame. if not specified will fetch it from the url prop */
    frame?: Frame;
    /** a function to handle mint buttons */
    onMint?: (t: OnMintArgs) => void;
    /** a function to handle transaction buttons, returns the transaction hash or null */
    onTransaction?: OnTransactionFunc;
    /** the context of this frame, used for generating Frame Action payloads */
    frameContext: FrameContext;
    /**
     * Extra data appended to the frame action payload
     */
    extraButtonRequestPayload?: Record<string, unknown>;
};
interface SignerStateInstance<T = object, B extends FrameActionBodyPayload = FrameActionBodyPayload> {
    signer?: T | null;
    hasSigner: boolean;
    signFrameAction: (actionContext: {
        target?: string;
        frameButton: FrameButton;
        buttonIndex: number;
        url: string;
        inputText?: string;
        signer: T | null;
        state?: string;
        transactionId?: `0x${string}`;
        frameContext: FrameContext;
    }) => Promise<{
        body: B;
        searchParams: URLSearchParams;
    }>;
    /** isLoading frame */
    isLoading?: null | FrameStackPending;
    /** is loading the signer */
    isLoadingSigner?: boolean;
    /** A function called when a frame button is clicked without a signer */
    onSignerlessFramePress: () => void;
    logout?: () => void;
}
type FrameRequest = {
    method: "GET";
    url: string;
} | {
    method: "POST";
    request: {
        body: object;
        searchParams: URLSearchParams;
    };
    url: string;
};
type FrameStackPending = {
    timestamp: Date;
} & FrameRequest;
type FrameStackBase = FrameStackPending & {
    /** speed in seconds */
    speed: number;
    responseStatus: number;
};
type FrameStackSuccess = FrameStackBase & {
    frame: Frame;
    frameValidationErrors: null | Record<string, string[]>;
    isValid: boolean;
};
type FrameStackError = (FrameStackBase | FrameStackSuccess) & {
    requestError: unknown;
};
type FramesStack = (FrameStackSuccess | FrameStackError)[];
type FrameState = {
    fetchFrame: (request: FrameRequest) => void | Promise<void>;
    clearFrameStack: () => void;
    /** The frame at the top of the stack (at index 0) */
    frame: Frame | null;
    /** A stack of frames with additional context, with the most recent frame at index 0 */
    framesStack: FramesStack;
    /** isLoading frame */
    isLoading?: null | FrameStackPending;
    inputText: string;
    setInputText: (s: string) => void;
    onButtonPress: (frameButton: FrameButton, index: number) => void | Promise<void>;
    /** Whether the frame at the top of the stack has any frame validation errors. Undefined when the frame is not loaded or set */
    isFrameValid: boolean | undefined;
    frameValidationErrors: Record<string, string[]> | undefined | null;
    error: unknown;
    homeframeUrl: string | null;
};
type OnMintArgs = {
    target: string;
    frameButton: FrameButton;
    frame: Frame;
};
type OnTransactionArgs = {
    transactionData: TransactionTargetResponse;
    frameButton: FrameButton;
    frame: Frame;
};
declare const themeParams: readonly ["bg", "buttonColor", "buttonBg", "buttonBorderColor", "buttonRadius", "buttonHoverBg"];
type FrameTheme = Partial<Record<(typeof themeParams)[number], string>>;
type FrameActionBodyPayload = Record<string, unknown>;
type FrameContext = FarcasterFrameContext;

export { type FrameContext as F, type OnTransactionFunc as O, type SignerStateInstance as S, type UseFrameReturn as U, type FrameRequest as a, type FrameStackPending as b, type FrameStackBase as c, type FrameStackSuccess as d, type FrameStackError as e, type FramesStack as f, type FrameState as g, type OnMintArgs as h, type OnTransactionArgs as i, type FrameTheme as j, type FrameActionBodyPayload as k, type FarcasterFrameContext as l, createFrameActionMessageWithSignerKey as m, type FarcasterSignerState as n, type FarcasterSigner as o, mockFarcasterSigner as p, signFrameAction as s, themeParams as t };
